class TestSettingsFactory:
    def test_create_settings_by_loader(self):
        assert False

    def test_create(self):
        assert False

    def test_setup_log(self):
        assert False

    def test_default(self):
        assert False
